// $Id: alsa_device.cc,v 1.15 2003/03/11 19:40:42 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//


#include "alsa_device.h"

//----------------------------------------------------------------------
AlsaDevice::AlsaDevice(snd_mixer_elem_t* element, const string& name)
  throw(InvalidChannelException)
  : Device(true, !snd_mixer_selem_is_playback_mono(element), 
	   snd_mixer_selem_has_capture_switch(element), 
	   false, name),
    element_(element), capture_enabled_(false)
{
  if(!snd_mixer_selem_has_capture_volume(element) &&
     !snd_mixer_selem_has_playback_volume(element))
  {
    string error(name);
    //    throw InvalidChannelException(error+" - channel not supported");
    is_supported_ = false;
    //    std::cout << "*** Not choosing " << name << std::endl;
    return;
    //    throw InvalidChannelException("channel not supported");
  }

    
//       if(snd_mixer_selem_has_capture_switch(element))
//       std::cout << name << " has capture switch" << std::endl;
//       else
//       std::cout << name << " doesn't have capture switch" << std::endl;

//       if(snd_mixer_selem_is_playback_mono(element))
//       std::cout << name << " is playback mono" << std::endl;
//       else
//       std::cout << name << " is playback stereo" << std::endl;

//       if(snd_mixer_selem_is_capture_mono(element))
//       std::cout << name << " is capture mono" << std::endl;
//       else
//       std::cout << name << " is capture stereo" << std::endl;

//       if(snd_mixer_selem_has_playback_volume(element))
//       std::cout << name << " has playback volume" << std::endl;
//       else
//       std::cout << name << " doesn't have playback volume" << std::endl;

//       if(snd_mixer_selem_has_capture_volume(element))
//       std::cout << name << " has capture volume" << std::endl;
//       else
//       std::cout << name << " doesn't have capture volume" << std::endl;

//       if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_FRONT_LEFT))
//       std::cout << name << " has left channel" << std::endl;
//       else
//       std::cout << name << " doesn't have left channel" << std::endl;

//       if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_FRONT_RIGHT))
//       std::cout << name << " has right channel" << std::endl;
//       else
//       std::cout << name << " doesn't have right channel" << std::endl;
  
  if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_FRONT_LEFT))
  {
    left_channel_ = SND_MIXER_SCHN_FRONT_LEFT;
    if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_FRONT_RIGHT))
      right_channel_ = SND_MIXER_SCHN_FRONT_RIGHT;
    else
      right_channel_ = SND_MIXER_SCHN_UNKNOWN;
  }
  else if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_REAR_LEFT))
  {
    left_channel_  = SND_MIXER_SCHN_REAR_LEFT;
    if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_REAR_RIGHT))
      right_channel_ = SND_MIXER_SCHN_REAR_RIGHT;
    else
      right_channel_ = SND_MIXER_SCHN_UNKNOWN;
  }
  else if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_FRONT_CENTER))
  {
    left_channel_ = SND_MIXER_SCHN_FRONT_CENTER;
    right_channel_ = SND_MIXER_SCHN_UNKNOWN;
  }
  else if(snd_mixer_selem_has_playback_channel(element, SND_MIXER_SCHN_WOOFER))
  {
    left_channel_ = SND_MIXER_SCHN_WOOFER;
    right_channel_ = SND_MIXER_SCHN_UNKNOWN;
  }
  else
  {
    left_channel_ = SND_MIXER_SCHN_UNKNOWN;
    right_channel_ = SND_MIXER_SCHN_UNKNOWN;
    //    std::cout << "*** " << name << " is not supported" << std::endl;
    is_supported_ = false;
    // FIXXME
    //       char error[256];
    //       sprintf(error, "%s - channel not supported", name);
    //      throw InvalidChannelException(error);

    //     string error(name);
    //     throw InvalidChannelException(error+" - channel not supported");
  }
  
  if(!snd_mixer_selem_has_playback_channel(element, right_channel_))
  {
    left_channel_  = SND_MIXER_SCHN_MONO;
    right_channel_ = SND_MIXER_SCHN_UNKNOWN;
  }
  
}

//----------------------------------------------------------------------
AlsaDevice::~AlsaDevice()
{
}

//----------------------------------------------------------------------
void AlsaDevice::setLeftValue(unsigned value)
{
  left_value_ = value;

  if(left_channel_ == SND_MIXER_SCHN_UNKNOWN)
    return;

  long vmin, vmax, vleft;

  if(!capture_enabled_)
  {
    snd_mixer_selem_get_playback_volume_range(element_, &vmin, &vmax);
    vleft = normalize(left_value_, 0, 100, vmin, vmax);
    snd_mixer_selem_set_playback_volume(element_, left_channel_, vleft);
  }
  else
  {
    snd_mixer_selem_get_capture_volume_range(element_, &vmin, &vmax);
    vleft = normalize(left_value_, 0, 100, vmin, vmax);
    snd_mixer_selem_set_capture_volume(element_, left_channel_, left_value_);
  }
}

//----------------------------------------------------------------------
void AlsaDevice::setRightValue(unsigned value)
{
  right_value_ = value;

  if(right_channel_ == SND_MIXER_SCHN_UNKNOWN)
    return;

  long vmin, vmax, vright;

  if(!capture_enabled_)
  {
    snd_mixer_selem_get_playback_volume_range(element_, &vmin, &vmax);
    vright = normalize(right_value_, 0, 100, vmin, vmax);
    snd_mixer_selem_set_playback_volume(element_, right_channel_, vright);
  }
  else
  {
    snd_mixer_selem_get_capture_volume_range(element_, &vmin, &vmax);
    vright = normalize(right_value_, 0, 100, vmin, vmax);
    snd_mixer_selem_set_capture_volume(element_, right_channel_, right_value_);
  }
}

//----------------------------------------------------------------------
unsigned AlsaDevice::getLeftValue() const
{
  long vmin, vmax, vleft;

  if(!capture_enabled_)
  {
    snd_mixer_selem_get_playback_volume_range(element_, &vmin, &vmax);
    snd_mixer_selem_get_playback_volume(element_, left_channel_, &vleft);  
    return normalize(vleft, vmin, vmax, 0, 100);
    //      left_value_ = normalize(vleft, vmin, vmax, 0, 100);
  }
  else
  {
    snd_mixer_selem_get_capture_volume_range(element_, &vmin, &vmax);
    snd_mixer_selem_get_capture_volume(element_, left_channel_, &vleft);  
    //      left_value_ = normalize(vleft, vmin, vmax, 0, 100);
    return normalize(vleft, vmin, vmax, 0, 100);
  }
  
  return left_value_;
}

//----------------------------------------------------------------------
unsigned AlsaDevice::getRightValue() const
{
  long vmin, vmax, vright;

  
  if (right_channel_ != SND_MIXER_SCHN_UNKNOWN)
  {
    if(!capture_enabled_)
    {
      snd_mixer_selem_get_playback_volume_range(element_, &vmin, &vmax);
      snd_mixer_selem_get_playback_volume(element_, right_channel_, &vright);
      //	  right_value_ = normalize(vright, vmin, vmax, 0, 100);
      return normalize(vright, vmin, vmax, 0, 100);
    }
    else
    {
      snd_mixer_selem_get_capture_volume_range(element_, &vmin, &vmax);
      snd_mixer_selem_get_capture_volume(element_, right_channel_, &vright);
      //      right_value_ = normalize(vright, vmin, vmax, 0, 100);
      return normalize(vright, vmin, vmax, 0, 100);
    }
  }
  
  return right_value_;
}

//----------------------------------------------------------------------
void AlsaDevice::mute()
{
  Device::mute();
  setLeftValue(0);
  setRightValue(0);
}

//----------------------------------------------------------------------
void AlsaDevice::unMute()
{
  setLeftValue(left_muted_);
  setRightValue(right_muted_);
  Device::unMute();
}


//----------------------------------------------------------------------
bool AlsaDevice::toggleCapture()
{
  int sw;

  if (snd_mixer_selem_has_capture_switch(element_))
  {
    if (snd_mixer_selem_has_capture_switch_joined(element_)) 
    {
      snd_mixer_selem_get_capture_switch(element_, left_channel_, &sw);
      snd_mixer_selem_set_capture_switch_all(element_, !sw);
    } 
    else
    {
      if (snd_mixer_selem_has_capture_channel(element_, left_channel_))
      {
	snd_mixer_selem_get_capture_switch(element_, left_channel_, &sw);
	snd_mixer_selem_set_capture_switch(element_, left_channel_, !sw);
      }
      if (right_channel_ != SND_MIXER_SCHN_UNKNOWN &&
	  snd_mixer_selem_has_capture_channel(element_, right_channel_))
      {
	snd_mixer_selem_get_capture_switch(element_, right_channel_, &sw);
	snd_mixer_selem_set_capture_switch(element_, right_channel_, !sw);
      }
    }
  }
  
  return !sw;
}

//----------------------------------------------------------------------
bool AlsaDevice::getCapture() const
{
  int sw;

  if (snd_mixer_selem_has_capture_switch(element_))
  {
    if (snd_mixer_selem_has_capture_switch_joined(element_)) 
    {
      snd_mixer_selem_get_capture_switch(element_, left_channel_, &sw);
      return (bool)sw;
    } 
    else
    {
      if (snd_mixer_selem_has_capture_channel(element_, left_channel_))
      {
	snd_mixer_selem_get_capture_switch(element_, left_channel_, &sw);
	return (bool)sw;
      }
    }
  }
  
  return false;
}

//----------------------------------------------------------------------
bool AlsaDevice::isPlayback() const
{
  return !capture_enabled_;
}

//----------------------------------------------------------------------
bool AlsaDevice::canCapture() const
{
  return snd_mixer_selem_has_capture_switch(element_);
}

//----------------------------------------------------------------------
snd_mixer_elem_t* AlsaDevice::getElement()
{
  return element_;
}

//----------------------------------------------------------------------
int AlsaDevice::normalize(int val, int omin, int omax, int nmin, int nmax) const
{
  int orange = omax - omin, nrange = nmax - nmin;
  int return_val;
  if (orange == 0)
    return 0;
  return_val = ((nrange * (val - omin)) + (orange / 2)) / orange + nmin;
  if(return_val > nmax)
    return nmax;
  else
    return return_val;
}

