// $Id: config_file.cc,v 1.10 2003/02/09 19:28:00 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#include "config_file.h"

//--------------------------------------------------------------------
Section::Section()
{
}

//--------------------------------------------------------------------
Section::~Section()
{
  parameters_.clear();
}

//--------------------------------------------------------------------
const char* Section::getStringParameter(const char* parameter) const
{
  elements_t::const_iterator i = parameters_.find(parameter);

  if (i == parameters_.end())
  {
    //      std::cout << "not found" << std::endl;
    return NULL;
  }
  else
  {
    return (*i).second.c_str();
  }
}

//--------------------------------------------------------------------
int Section::getIntParameter(const char* parameter) const
{
  const char* dummy = getStringParameter(parameter);
  if(dummy != NULL)
  {
    return atoi(dummy);
  }
  return 0;
}


//--------------------------------------------------------------------
void Section::addParameter(const char* parameter, const char* val)
{
  string key = parameter, value = val;

  parameters_.insert (pair<string, string>(key, value));
}

//====================================================================


//--------------------------------------------------------------------
ConfigFile::ConfigFile()
  : current_section_(0)
{
}

//--------------------------------------------------------------------
ConfigFile::~ConfigFile()
{
  sections_.clear();
  delete[] current_section_;
}

//-------------------------------------------------------------------
void ConfigFile::addSection(const char* section)
{
  delete[] current_section_;

  current_section_ = new char[strlen(section)+1];
  strcpy(current_section_, section);

  //  Not needed
  //  sections_.insert (make_pair (section, Section()));
  //  sections_[section] = Section();

}


//-------------------------------------------------------------------
const char* ConfigFile::getStringParameter(const char* section, const char* parameter) const
{
  sections_t::const_iterator i = sections_.find(section);

  if (i == sections_.end())
  {
    //      cout << "Section '" << section << "' not found" << endl;
    return NULL;
  }
  else
  {
    return (*i).second.getStringParameter(parameter);
  }
}

//-------------------------------------------------------------------
int ConfigFile::getIntParameter(const char* section, const char* parameter) const
{
  sections_t::const_iterator i = sections_.find(section);

  if (i == sections_.end())
  {
    //      cout << "Section '" << section << "' not found" << endl;
    return 0;
  }
  else
  {
    return (*i).second.getIntParameter(parameter);
  }
  
}

//-------------------------------------------------------------------
void ConfigFile::addParameter(const char* section, const char* parameter, const char* value)
{
  sections_[section].addParameter(parameter, value);
}


//-------------------------------------------------------------------
// void ConfigFile::delParameter(const char* section, const char* parameter)
// {
//  delete sections_[section][parameter];
//  map:erase(sections_[section][parameter]);
// }
  


//-------------------------------------------------------------------
void ConfigFile::dumpSections() const
{
  cout << "Sections:" << endl;
  for(sections_t::const_iterator i = sections_.begin(); i != sections_.end(); i++)
  {
    cout << "  " << i->first << endl;      
  }
}
