// $Id: theme.cc,v 1.12 2003/03/11 19:40:42 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#include "theme.h"

//--------------------------------------------------------------------
Theme::Theme(const string& filename)
  throw(InvalidThemeException)
{
  ConfigReader* config;

  try
  {
    config = new ConfigReader(filename);
    initTheme(config);
  }
  catch(FileNotFoundException& e)
  {
    throw InvalidThemeException("Could not open theme-file");
  }
  catch(InvalidThemeException& e)
  {
    throw e;
  }


  delete config;
  config = NULL;

}

//--------------------------------------------------------------------
Theme::Theme(ConfigReader* config_reader)
  throw(InvalidThemeException)
{
  try
  {
    initTheme(config_reader);
  }
  catch(InvalidThemeException& e)
  {
    throw e;
  }
}

//--------------------------------------------------------------------
// TODO: verify values and throw exception
void Theme::initTheme(ConfigReader* config) 
  throw(InvalidThemeException)
{
  bar_x_          = config->getIntParameter("bar", "x");

  bar_y_          = config->getIntParameter("bar", "y");

  bar_width_      = config->getIntParameter("bar", "width");
  if(bar_width_ == 0)
    throw InvalidThemeException("Theme file "+config->getFileName()+" is not valid");

  bar_height_     = config->getIntParameter("bar", "height");
  bar_alignment_  = config->getIntParameter("bar", "alignment");
  bar_thickness_  = config->getIntParameter("bar", "thickness");
  bar_num_        = config->getIntParameter("bar", "num");
  if(bar_num_ == 0)
    throw InvalidThemeException("Theme file "+config->getFileName()+" is not valid");

  bar_color_high_name_ = config->getStringParameter("bar", "color_high");
  bar_color_low_name_  = config->getStringParameter("bar", "color_low");
  bar_color_bg_name_   = config->getStringParameter("bar", "color_background");

  // TODO: some extra sanity checking
  bar_x2_  = bar_x_ + bar_width_;
  bar_y2_  = bar_y_ + bar_height_;
  bar_gap_ = (bar_height_ - (bar_num_ * bar_thickness_)) / bar_num_;

  icon_x_          = config->getIntParameter("icon", "x");
  icon_y_          = config->getIntParameter("icon", "y");
  icon_width_      = config->getIntParameter("icon", "width");
  icon_height_     = config->getIntParameter("icon", "height");
  icon_image_name_ = config->getStringParameter("icon", "image");

  label_x_          = config->getIntParameter("label", "x");
  label_y_          = config->getIntParameter("label", "y");
  label_width_      = config->getIntParameter("label", "width");
  label_height_     = config->getIntParameter("label", "height");
  label_font_name_  = config->getStringParameter("label", "font");
  label_font_width_ = config->getIntParameter("label", "font_width");
  label_font_height_= config->getIntParameter("label", "font_height");

  button_left_x_          = config->getIntParameter("button_left", "x");
  button_left_y_          = config->getIntParameter("button_left", "y");
  button_left_width_      = config->getIntParameter("button_left", "width");
  button_left_height_     = config->getIntParameter("button_left", "height");
  button_left_image_name_ = config->getStringParameter("button_left", "image");

  button_right_x_       = config->getIntParameter("button_right", "x");
  button_right_y_       = config->getIntParameter("button_right", "y");
  button_right_width_   = config->getIntParameter("button_right", "width");
  button_right_height_  = config->getIntParameter("button_right", "height");
  button_right_image_name_   = config->getStringParameter("button_right", "image");

  button_mute_x_       = config->getIntParameter("button_mute", "x");
  button_mute_y_       = config->getIntParameter("button_mute", "y");
  button_mute_width_   = config->getIntParameter("button_mute", "width");
  button_mute_height_  = config->getIntParameter("button_mute", "height");
  button_mute_image_name_   = config->getStringParameter("button_mute", "image");

  button_rec_x_       = config->getIntParameter("button_rec", "x");
  button_rec_y_       = config->getIntParameter("button_rec", "y");
  button_rec_width_   = config->getIntParameter("button_rec", "width");
  button_rec_height_  = config->getIntParameter("button_rec", "height");
  button_rec_image_name_   = config->getStringParameter("button_rec", "image");

  background_image_name_ = config->getStringParameter("background", "image");
}


//--------------------------------------------------------------------
Theme::~Theme()
{
  delete background_image_;
  delete[] icon_images_;
  delete button_left_image_;
  delete button_right_image_;
  delete button_mute_image_;
  delete button_rec_image_;

  delete label_font_;

  delete[] bar_colors_;

  delete[] background_image_name_;
  delete[] icon_image_name_;
  delete[] button_left_image_name_;
  delete[] button_right_image_name_;
  delete[] button_mute_image_name_;
  delete[] button_rec_image_name_;

  delete[] label_font_name_;

  delete[]bar_color_high_name_;
  delete[]bar_color_low_name_;
  delete[]bar_color_bg_name_;
}

//--------------------------------------------------------------------
void Theme::initImages(XHandler* x_handler)
  throw(InvalidThemeException)
{
  unsigned icon_w = 0, icon_h = 0;

  if(background_image_name_ == NULL)
    throw InvalidThemeException("[background] image is missing");
  Pixmap *mask = new Pixmap;
  background_image_   = new Image(background_image_name_, x_handler, &icon_w, &icon_h, mask);
  x_handler->initMask(mask, icon_w, icon_h);
  //   if(icon_w != XHandler::PANEL_SIZE ||
  //      icon_h != XHandler::PANEL_SIZE)
  //     x_handler->setSize(icon_w, icon_h);

  if(icon_image_name_ == NULL)
    throw InvalidThemeException("[icon] image is missing");

  icon_w = 0;
  icon_h = 0;
  icon_images_       = new Image(icon_image_name_, x_handler, &icon_w, &icon_h, NULL);
  icon_num_ = icon_w / icon_width_;

  if(button_left_image_name_ == NULL)
    throw InvalidThemeException("[button_left] image is missing");
  button_left_image_  = new Image(button_left_image_name_, x_handler);

  if(button_right_image_name_ == NULL)
    throw InvalidThemeException("[button_right] image is missing");
  button_right_image_ = new Image(button_right_image_name_, x_handler);

  if(button_mute_image_name_ == NULL)
    throw InvalidThemeException("[button_mute] image is missing");
  button_mute_image_  = new Image(button_mute_image_name_, x_handler);

  if(button_rec_image_name_ == NULL)
    throw InvalidThemeException("[button_rec] image is missing");
  button_rec_image_   = new Image(button_rec_image_name_, x_handler);

  if(label_font_name_ == NULL)
    throw InvalidThemeException("[label] font is missing");
  label_font_         = new FontMap(label_font_name_, label_font_width_,
				    label_font_height_, x_handler);

  if(bar_color_bg_name_ == NULL)
    throw InvalidThemeException("[bar] color_background is missing");
  bar_color_inactive_ = new Color(bar_color_bg_name_, x_handler);

  if(bar_color_high_name_ == NULL)
    throw InvalidThemeException("[bar] color_high is missing");

  if(bar_color_low_name_ == NULL)
    throw InvalidThemeException("[bar] color_low is missing");
  
  if(bar_num_ <= 1)
    throw InvalidThemeException("[bar] num is invalid");
  
  bar_colors_ = new Color*[bar_num_];

  for(unsigned count=0; count<bar_num_; count++)
  {
    bar_colors_[count] = new Color(bar_color_high_name_, count*2, bar_color_low_name_, 100-count*4, x_handler);
  }

}
