// $Id: theme.h,v 1.8 2003/01/06 17:51:30 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

// TODO:
// Set default values of parameters are omitted

#ifndef __theme_h__
#define __theme_h__

#include <string>

#include "color.h"
#include "image.h"
#include "font_map.h"
#include "config_reader.h"
#include "exception.h"
#include "x_handler.h"

/* #define S_BAR          0 */
/* #define S_ICON         1 */
/* #define S_LABEL        2 */
/* #define S_COUNTER      3 */
/* #define S_BUTTON_LEFT  4 */
/* #define S_BUTTON_RIGHT 5 */
/* #define S_BUTTON_MUTE  6 */
/* #define S_BUTTON_REC   7 */
/* #define S_BACKGROUND   8 */

/* #define V_X            0 */
/* #define V_Y            1 */
/* #define V_WIDTH        2 */
/* #define V_HEIGHT       3 */
/* #define V_ALIGNMENT    4 */
/* #define V_FONT         5 */
/* #define V_IMAGE        6 */

#ifndef NULL
#define NULL 0
#endif

/* #define VERTICAL 0 */
/* #define HORIZONTAL 1 */

class Theme 
{
 protected:
  const char* background_image_name_;
  Image* background_image_;
  
  unsigned bar_x_;
  unsigned bar_y_;
  unsigned bar_width_;
  unsigned bar_height_;
  unsigned bar_alignment_;

  unsigned bar_x2_;
  unsigned bar_y2_;

  unsigned bar_gap_;

  unsigned bar_thickness_;
  unsigned bar_num_;
  Color** bar_colors_;
  Color* bar_color_inactive_;
  const char* bar_color_high_name_;
  const char* bar_color_low_name_;
  const char* bar_color_bg_name_;

  unsigned icon_x_;
  unsigned icon_y_;  
  unsigned icon_width_;
  unsigned icon_height_;
  unsigned icon_num_;
  const char* icon_image_name_;
  Image* icon_images_;

  unsigned label_x_;
  unsigned label_y_;
  unsigned label_width_;
  unsigned label_height_;
  const char* label_font_name_;
  FontMap* label_font_;
  unsigned label_font_width_;
  unsigned label_font_height_;

  unsigned button_left_x_;
  unsigned button_left_y_;
  unsigned button_left_width_;
  unsigned button_left_height_;
  const char* button_left_image_name_;
  Image* button_left_image_;

  unsigned button_right_x_;
  unsigned button_right_y_;
  unsigned button_right_width_;
  unsigned button_right_height_;
  const char* button_right_image_name_;
  Image* button_right_image_;

  unsigned button_mute_x_;
  unsigned button_mute_y_;
  unsigned button_mute_width_;
  unsigned button_mute_height_;
  Image* button_mute_image_;
  const char* button_mute_image_name_;

  unsigned button_rec_x_;
  unsigned button_rec_y_;
  unsigned button_rec_width_;
  unsigned button_rec_height_;
  Image* button_rec_image_;
  const char* button_rec_image_name_;

  void initImages(XHandler*)
    throw(InvalidThemeException);
  
  void initTheme(ConfigReader*)
    throw(InvalidThemeException);
  
  static const unsigned char S_BAR  = 0;
  static const unsigned char S_ICON = 1;
  static const unsigned char S_LABEL = 2;
  static const unsigned char S_COUNTER = 3;
  static const unsigned char S_BUTTON_LEFT =  4;
  static const unsigned char S_BUTTON_RIGHT = 5;
  static const unsigned char S_BUTTON_MUTE = 6;
  static const unsigned char S_BUTTON_REC =  7;
  static const unsigned char S_BACKGROUND = 8;

  static const unsigned char V_X =  0;
  static const unsigned char V_Y =  1;
  static const unsigned char V_WIDTH = 2;
  static const unsigned char V_HEIGHT = 3;
  static const unsigned char V_ALIGNMENT = 4;
  static const unsigned char V_FONT = 5;
  static const unsigned char V_IMAGE = 6;

  static const unsigned char VERTICAL = 0;
  static const unsigned char HORIZONTAL = 1;

 public:
  Theme(const string& filename) throw(InvalidThemeException);
  Theme(ConfigReader* config_reader) throw(InvalidThemeException);
  virtual ~Theme();

  friend class DockApp;
};

#endif //__theme_h__
