// $Id: x_handler.h,v 1.12 2003/02/09 19:28:00 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
// Parts copyright (C) 1998  Sam Hawker <shawkie@geocities.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#ifndef __x_handler_h__
#define __x_handler_h__

#include <iostream>
#include <string>

#include <stdlib.h>

// X-Windows includes - standard
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

// User defines - standard
/* #define WINDOWMAKER false */
/* #define USESHAPE    false */
/* #define AFTERSTEP   false */
#define NORMSIZE    64
#define ASTEPSIZE   56
#define NAME        "wmmixer"
#define CLASS       "WMMixer"

//#include "wmmixer.h"

using std::string;

class XHandler 
{
 protected:
  int window_size_;
  int screen_;

  bool is_windowed_;
  bool is_wmaker_;
  bool is_ushape_;
  bool is_astep_;
  Display* display_;

  GC graphics_context_;

  Pixmap pixmap_display_; // window pixmap
  Pixmap *pixmap_mask_;    // mask pixmap for shape

  // static?
  Window window_icon_;
  Window window_main_;
  Window window_root_;

  Pixel back_pix_;
  Pixel fore_pix_;

  Atom _XA_GNUSTEP_WM_FUNC;
  Atom deleteWin;

  int offset_w_;
  int offset_h_;
  int width_;
  int height_;
  int depth_;

  void initGraphicsContext();
  void initWindow(int argc, char** argv);
  char* display_name_;
  char* position_name_;

  Pixmap createBgpanel();

 public:
  explicit XHandler(const string&, const string&, char);
  virtual ~XHandler();

  static const char WMAKER   = 0x1;
  static const char ASTEP    = 0x2;
  static const char USHAPED  = 0x4;
  static const char WINDOWED = 0x8;
  static const unsigned PANEL_SIZE = 64;

  void setDisplay(const string&);
  void initMask(Pixmap*, unsigned, unsigned);
  Display* getDisplay() const;
  Window getWindow() const;
  void setPosition(char *);
  void setWindowMaker();
  void setAfterStep();
  void setUnshaped();

  void drawRectangle(unsigned long color, unsigned x1, unsigned y1,
		     unsigned x2, unsigned y2);
  void drawShade(Pixmap*,unsigned, unsigned, unsigned, unsigned);
  void drawPixmap(Pixmap*,unsigned,unsigned,unsigned,unsigned);
  void drawPixmapOffset(Pixmap*,unsigned,unsigned,unsigned,unsigned,unsigned,unsigned);
  void init(int, char**);
  int  getWindowSize() const;
  Atom getDeleteWin() const;
  Pixmap* readPixmapFile(char*, unsigned* width = NULL, unsigned* height=NULL,
			 Pixmap* mask = NULL);

  void redraw();

  unsigned mixColor(const char *colorname1, int prop1, 
		    const char *colorname2, int prop2);
  unsigned getColor(const char *colorname) const;
  unsigned getOffsetX() const { return offset_w_; }
  unsigned getOffsetY() const { return offset_h_; }
};

#endif //__x_handler_h__
